@@ -46,9 +46,9 @@
 import {
     defaults,
     keep_name,
-    member,
     mergeSort,
     push_uniq,
+    make_node,
     return_false,
     return_this,
     return_true,
@@ -64,9 +64,11 @@ import {
     AST_Defun,
     AST_Destructuring,
     AST_Dot,
+    AST_DotHash,
     AST_Export,
     AST_For,
     AST_ForIn,
+    AST_ForOf,
     AST_Function,
     AST_Import,
     AST_IterationStatement,
@@ -81,6 +83,7 @@ import {
     AST_Sequence,
     AST_String,
     AST_Sub,
+    AST_Switch,
     AST_SwitchBranch,
     AST_Symbol,
     AST_SymbolBlockDeclaration,
@@ -98,38 +101,65 @@ import {
     AST_SymbolRef,
     AST_SymbolVar,
     AST_Toplevel,
+    AST_VarDef,
     AST_With,
     TreeWalker,
+    walk,
+    walk_abort
 } from "./ast.js";
 import {
-    RESERVED_WORDS,
+    ALL_RESERVED_WORDS,
     js_error,
 } from "./parse.js";
 
-function SymbolDef(scope, orig, init) {
-    this.name = orig.name;
-    this.orig = [ orig ];
-    this.init = init;
-    this.eliminated = 0;
-    this.scope = scope;
-    this.references = [];
-    this.replaced = 0;
-    this.global = false;
-    this.export = false;
-    this.mangled_name = null;
-    this.undeclared = false;
-    this.id = SymbolDef.next_id++;
-}
+const MASK_EXPORT_DONT_MANGLE = 1 << 0;
+const MASK_EXPORT_WANT_MANGLE = 1 << 1;
 
-SymbolDef.next_id = 1;
+let function_defs = null;
+let unmangleable_names = null;
+/**
+ * When defined, there is a function declaration somewhere that's inside of a block.
+ * See https://tc39.es/ecma262/multipage/additional-ecmascript-features-for-web-browsers.html#sec-block-level-function-declarations-web-legacy-compatibility-semantics
+*/
+let scopes_with_block_defuns = null;
 
-var MASK_EXPORT_DONT_MANGLE = 1 << 0;
-var MASK_EXPORT_WANT_MANGLE = 1 << 1;
-
-SymbolDef.prototype = {
-    unmangleable: function(options) {
+class SymbolDef {
+    constructor(scope, orig, init) {
+        this.name = orig.name;
+        this.orig = [ orig ];
+        this.init = init;
+        this.eliminated = 0;
+        this.assignments = 0;
+        this.scope = scope;
+        this.replaced = 0;
+        this.global = false;
+        this.export = 0;
+        this.mangled_name = null;
+        this.undeclared = false;
+        this.id = SymbolDef.next_id++;
+        this.chained = false;
+        this.direct_access = false;
+        this.escaped = 0;
+        this.recursive_refs = 0;
+        this.references = [];
+        this.should_replace = undefined;
+        this.single_use = false;
+        this.fixed = false;
+        Object.seal(this);
+    }
+    fixed_value() {
+        if (!this.fixed || this.fixed instanceof AST_Node) return this.fixed;
+        return this.fixed();
+    }
+    unmangleable(options) {
         if (!options) options = {};
 
+        if (
+            function_defs &&
+            function_defs.has(this.id) &&
+            keep_name(options.keep_fnames, this.orig[0].name)
+        ) return true;
+
         return this.global && !options.toplevel
             || (this.export & MASK_EXPORT_DONT_MANGLE)
             || this.undeclared
@@ -139,9 +169,9 @@ SymbolDef.prototype = {
             || this.orig[0] instanceof AST_SymbolMethod
             || (this.orig[0] instanceof AST_SymbolClass
                   || this.orig[0] instanceof AST_SymbolDefClass) && keep_name(options.keep_classnames, this.orig[0].name);
-    },
-    mangle: function(options) {
-        var cache = options.cache && options.cache.props;
+    }
+    mangle(options) {
+        const cache = options.cache && options.cache.props;
         if (this.global && cache && cache.has(this.name)) {
             this.mangled_name = cache.get(this.name);
         } else if (!this.mangled_name && !this.unmangleable(options)) {
@@ -149,58 +179,83 @@ SymbolDef.prototype = {
             var sym = this.orig[0];
             if (options.ie8 && sym instanceof AST_SymbolLambda)
                 s = s.parent_scope;
-            var def;
-            if (def = this.redefined()) {
-                this.mangled_name = def.mangled_name || def.name;
-            } else
-                this.mangled_name = s.next_mangled(options, this);
+            const redefinition = redefined_catch_def(this);
+            this.mangled_name = redefinition
+                ? redefinition.mangled_name || redefinition.name
+                : s.next_mangled(options, this);
             if (this.global && cache) {
                 cache.set(this.name, this.mangled_name);
             }
         }
-    },
-    redefined: function() {
-        return this.defun && this.defun.variables.get(this.name);
     }
-};
+}
 
-AST_Toplevel.DEFMETHOD("figure_out_scope", function(options) {
+SymbolDef.next_id = 1;
+
+function redefined_catch_def(def) {
+    if (def.orig[0] instanceof AST_SymbolCatch
+        && def.scope.is_block_scope()
+    ) {
+        return def.scope.get_defun_scope().variables.get(def.name);
+    }
+}
+
+AST_Scope.DEFMETHOD("figure_out_scope", function(options, { parent_scope = null, toplevel = this } = {}) {
     options = defaults(options, {
         cache: null,
         ie8: false,
         safari10: false,
     });
 
+    if (!(toplevel instanceof AST_Toplevel)) {
+        throw new Error("Invalid toplevel scope");
+    }
+
     // pass 1: setup scope chaining and handle definitions
-    var self = this;
-    var scope = self.parent_scope = null;
+    var scope = this.parent_scope = parent_scope;
     var labels = new Map();
     var defun = null;
     var in_destructuring = null;
     var for_scopes = [];
-    var tw = new TreeWalker(function(node, descend) {
+    var tw = new TreeWalker((node, descend) => {
         if (node.is_block_scope()) {
-            var save_scope = scope;
+            const save_scope = scope;
             node.block_scope = scope = new AST_Scope(node);
+            scope._block_scope = true;
             scope.init_scope_vars(save_scope);
-            if (!(node instanceof AST_Scope)) {
-                scope.uses_with = save_scope.uses_with;
-                scope.uses_eval = save_scope.uses_eval;
-                scope.directives = save_scope.directives;
-            }
+            scope.uses_with = save_scope.uses_with;
+            scope.uses_eval = save_scope.uses_eval;
+
             if (options.safari10) {
-                if (node instanceof AST_For || node instanceof AST_ForIn) {
+                if (node instanceof AST_For || node instanceof AST_ForIn || node instanceof AST_ForOf) {
                     for_scopes.push(scope);
                 }
             }
-            descend();
+
+            if (node instanceof AST_Switch) {
+                // XXX: HACK! Ensure the switch expression gets the correct scope (the parent scope) and the body gets the contained scope
+                // AST_Switch has a scope within the body, but it itself "is a block scope"
+                // This means the switched expression has to belong to the outer scope
+                // while the body inside belongs to the switch itself.
+                // This is pretty nasty and warrants an AST change
+                const the_block_scope = scope;
+                scope = save_scope;
+                node.expression.walk(tw);
+                scope = the_block_scope;
+                for (let i = 0; i < node.body.length; i++) {
+                    node.body[i].walk(tw);
+                }
+            } else {
+                descend();
+            }
             scope = save_scope;
             return true;
         }
         if (node instanceof AST_Destructuring) {
-            in_destructuring = node;  // These don't nest
+            const save_destructuring = in_destructuring;
+            in_destructuring = node;
             descend();
-            in_destructuring = null;
+            in_destructuring = save_destructuring;
             return true;
         }
         if (node instanceof AST_Scope) {
@@ -246,7 +301,14 @@ AST_Toplevel.DEFMETHOD("figure_out_scope", function(options) {
             // scope when we encounter the AST_Defun node (which is
             // instanceof AST_Scope) but we get to the symbol a bit
             // later.
-            mark_export((node.scope = defun.parent_scope.get_defun_scope()).def_function(node, defun), 1);
+            const closest_scope = defun.parent_scope;
+
+            // In strict mode, function definitions are block-scoped
+            node.scope = tw.directives["use strict"]
+                ? closest_scope
+                : closest_scope.get_defun_scope();
+
+            mark_export(node.scope.def_function(node, defun), 1);
         } else if (node instanceof AST_SymbolClass) {
             mark_export(defun.def_variable(node, defun), 1);
         } else if (node instanceof AST_SymbolImport) {
@@ -255,9 +317,12 @@ AST_Toplevel.DEFMETHOD("figure_out_scope", function(options) {
             // This deals with the name of the class being available
             // inside the class.
             mark_export((node.scope = defun.parent_scope).def_function(node, defun), 1);
-        } else if (node instanceof AST_SymbolVar
+        } else if (
+            node instanceof AST_SymbolVar
             || node instanceof AST_SymbolLet
-            || node instanceof AST_SymbolConst) {
+            || node instanceof AST_SymbolConst
+            || node instanceof AST_SymbolCatch
+        ) {
             var def;
             if (node instanceof AST_SymbolBlockDeclaration) {
                 def = scope.def_variable(node, null);
@@ -272,7 +337,7 @@ AST_Toplevel.DEFMETHOD("figure_out_scope", function(options) {
                 return !(sym instanceof AST_SymbolLet || sym instanceof AST_SymbolConst);
             })) {
                 js_error(
-                    node.name + " redeclared",
+                    `"${node.name}" is redeclared`,
                     node.start.file,
                     node.start.line,
                     node.start.col,
@@ -280,17 +345,14 @@ AST_Toplevel.DEFMETHOD("figure_out_scope", function(options) {
                 );
             }
             if (!(node instanceof AST_SymbolFunarg)) mark_export(def, 2);
-            def.destructuring = in_destructuring;
             if (defun !== scope) {
-                node.mark_enclosed(options);
+                node.mark_enclosed();
                 var def = scope.find_variable(node);
                 if (node.thedef !== def) {
                     node.thedef = def;
-                    node.reference(options);
+                    node.reference();
                 }
             }
-        } else if (node instanceof AST_SymbolCatch) {
-            scope.def_variable(node).defun = defun;
         } else if (node instanceof AST_LabelRef) {
             var sym = labels.get(node.name);
             if (!sym) throw new Error(string_template("Undefined label {name} [{line},{col}]", {
@@ -302,35 +364,39 @@ AST_Toplevel.DEFMETHOD("figure_out_scope", function(options) {
         }
         if (!(scope instanceof AST_Toplevel) && (node instanceof AST_Export || node instanceof AST_Import)) {
             js_error(
-                node.TYPE + " statement may only appear at top level",
+                `"${node.TYPE}" statement may only appear at the top level`,
                 node.start.file,
                 node.start.line,
                 node.start.col,
                 node.start.pos
             );
         }
+    });
+    this.walk(tw);
 
-        function mark_export(def, level) {
-            if (in_destructuring) {
-                var i = 0;
-                do {
-                    level++;
-                } while (tw.parent(i++) !== in_destructuring);
-            }
-            var node = tw.parent(level);
-            if (def.export = node instanceof AST_Export && MASK_EXPORT_DONT_MANGLE) {
-                var exported = node.exported_definition;
-                if ((exported instanceof AST_Defun || exported instanceof AST_DefClass) && node.is_default) {
-                    def.export = MASK_EXPORT_WANT_MANGLE;
-                }
+    function mark_export(def, level) {
+        if (in_destructuring) {
+            var i = 0;
+            do {
+                level++;
+            } while (tw.parent(i++) !== in_destructuring);
+        }
+        var node = tw.parent(level);
+        if (def.export = node instanceof AST_Export ? MASK_EXPORT_DONT_MANGLE : 0) {
+            var exported = node.exported_definition;
+            if ((exported instanceof AST_Defun || exported instanceof AST_DefClass) && node.is_default) {
+                def.export = MASK_EXPORT_WANT_MANGLE;
             }
         }
-    });
-    self.walk(tw);
+    }
 
     // pass 2: find back references and eval
-    self.globals = new Map();
-    var tw = new TreeWalker(function(node, descend) {
+    const is_toplevel = this instanceof AST_Toplevel;
+    if (is_toplevel) {
+        this.globals = new Map();
+    }
+
+    var tw = new TreeWalker(node => {
         if (node instanceof AST_LoopControl && node.label) {
             node.label.thedef.references.push(node);
             return true;
@@ -345,13 +411,14 @@ AST_Toplevel.DEFMETHOD("figure_out_scope", function(options) {
             var sym;
             if (tw.parent() instanceof AST_NameMapping && tw.parent(1).module_name
                 || !(sym = node.scope.find_variable(name))) {
-                sym = self.def_global(node);
+
+                sym = toplevel.def_global(node);
                 if (node instanceof AST_SymbolExport) sym.export = MASK_EXPORT_DONT_MANGLE;
             } else if (sym.scope instanceof AST_Lambda && name == "arguments") {
-                sym.scope.uses_arguments = true;
+                sym.scope.get_defun_scope().uses_arguments = true;
             }
             node.thedef = sym;
-            node.reference(options);
+            node.reference();
             if (node.scope.is_block_scope()
                 && !(sym.orig[0] instanceof AST_SymbolBlockDeclaration)) {
                 node.scope = node.scope.get_defun_scope();
@@ -360,7 +427,7 @@ AST_Toplevel.DEFMETHOD("figure_out_scope", function(options) {
         }
         // ensure mangling works if catch reuses a scope variable
         var def;
-        if (node instanceof AST_SymbolCatch && (def = node.definition().redefined())) {
+        if (node instanceof AST_SymbolCatch && (def = redefined_catch_def(node.definition()))) {
             var s = node.scope;
             while (s) {
                 push_uniq(s.enclosed, def);
@@ -369,25 +436,27 @@ AST_Toplevel.DEFMETHOD("figure_out_scope", function(options) {
             }
         }
     });
-    self.walk(tw);
+    this.walk(tw);
 
     // pass 3: work around IE8 and Safari catch scope bugs
     if (options.ie8 || options.safari10) {
-        self.walk(new TreeWalker(function(node, descend) {
+        walk(this, node => {
             if (node instanceof AST_SymbolCatch) {
                 var name = node.name;
                 var refs = node.thedef.references;
-                var scope = node.thedef.defun;
-                var def = scope.find_variable(name) || self.globals.get(name) || scope.def_variable(node);
+                var scope = node.scope.get_defun_scope();
+                var def = scope.find_variable(name)
+                    || toplevel.globals.get(name)
+                    || scope.def_variable(node);
                 refs.forEach(function(ref) {
                     ref.thedef = def;
-                    ref.reference(options);
+                    ref.reference();
                 });
                 node.thedef = def;
-                node.reference(options);
+                node.reference();
                 return true;
             }
-        }));
+        });
     }
 
     // pass 4: add symbol definitions to loop scopes
@@ -395,8 +464,7 @@ AST_Toplevel.DEFMETHOD("figure_out_scope", function(options) {
     // https://github.com/mishoo/UglifyJS2/issues/1753
     // https://bugs.webkit.org/show_bug.cgi?id=171041
     if (options.safari10) {
-        for (var i = 0; i < for_scopes.length; i++) {
-            var scope = for_scopes[i];
+        for (const scope of for_scopes) {
             scope.parent_scope.variables.forEach(function(def) {
                 push_uniq(scope.enclosed, def);
             });
@@ -419,7 +487,6 @@ AST_Toplevel.DEFMETHOD("def_global", function(node) {
 
 AST_Scope.DEFMETHOD("init_scope_vars", function(parent_scope) {
     this.variables = new Map();         // map name to AST_SymbolVar (variables defined in this scope; includes functions)
-    this.functions = new Map();         // map name to AST_SymbolDefun (functions defined in this scope)
     this.uses_with = false;             // will be set to true if this or some nested scope uses the `with` statement
     this.uses_eval = false;             // will be set to true if this or nested scope uses the global `eval`
     this.parent_scope = parent_scope;   // the parent scope
@@ -427,12 +494,138 @@ AST_Scope.DEFMETHOD("init_scope_vars", function(parent_scope) {
     this.cname = -1;                    // the current index for mangling functions/variables
 });
 
+AST_Scope.DEFMETHOD("conflicting_def", function (name) {
+    return (
+        this.enclosed.find(def => def.name === name)
+        || this.variables.has(name)
+        || (this.parent_scope && this.parent_scope.conflicting_def(name))
+    );
+});
+
+AST_Scope.DEFMETHOD("conflicting_def_shallow", function (name) {
+    return (
+        this.enclosed.find(def => def.name === name)
+        || this.variables.has(name)
+    );
+});
+
+AST_Scope.DEFMETHOD("add_child_scope", function (scope) {
+    // `scope` is going to be moved into `this` right now.
+    // Update the required scopes' information
+
+    if (scope.parent_scope === this) return;
+
+    scope.parent_scope = this;
+
+    // Propagate to this.uses_arguments from arrow functions
+    if ((scope instanceof AST_Arrow) && !this.uses_arguments) {
+        this.uses_arguments = walk(scope, node => {
+            if (
+                node instanceof AST_SymbolRef
+                && node.scope instanceof AST_Lambda
+                && node.name === "arguments"
+            ) {
+                return walk_abort;
+            }
+
+            if (node instanceof AST_Lambda && !(node instanceof AST_Arrow)) {
+                return true;
+            }
+        });
+    }
+
+    this.uses_with = this.uses_with || scope.uses_with;
+    this.uses_eval = this.uses_eval || scope.uses_eval;
+
+    const scope_ancestry = (() => {
+        const ancestry = [];
+        let cur = this;
+        do {
+            ancestry.push(cur);
+        } while ((cur = cur.parent_scope));
+        ancestry.reverse();
+        return ancestry;
+    })();
+
+    const new_scope_enclosed_set = new Set(scope.enclosed);
+    const to_enclose = [];
+    for (const scope_topdown of scope_ancestry) {
+        to_enclose.forEach(e => push_uniq(scope_topdown.enclosed, e));
+        for (const def of scope_topdown.variables.values()) {
+            if (new_scope_enclosed_set.has(def)) {
+                push_uniq(to_enclose, def);
+                push_uniq(scope_topdown.enclosed, def);
+            }
+        }
+    }
+});
+
+function find_scopes_visible_from(scopes) {
+    const found_scopes = new Set();
+
+    for (const scope of new Set(scopes)) {
+        (function bubble_up(scope) {
+            if (scope == null || found_scopes.has(scope)) return;
+
+            found_scopes.add(scope);
+
+            bubble_up(scope.parent_scope);
+        })(scope);
+    }
+
+    return [...found_scopes];
+}
+
+// Creates a symbol during compression
+AST_Scope.DEFMETHOD("create_symbol", function(SymClass, {
+    source,
+    tentative_name,
+    scope,
+    conflict_scopes = [scope],
+    init = null
+} = {}) {
+    let symbol_name;
+
+    conflict_scopes = find_scopes_visible_from(conflict_scopes);
+
+    if (tentative_name) {
+        // Implement hygiene (no new names are conflicting with existing names)
+        tentative_name =
+            symbol_name =
+            tentative_name.replace(/(?:^[^a-z_$]|[^a-z0-9_$])/ig, "_");
+
+        let i = 0;
+        while (conflict_scopes.find(s => s.conflicting_def_shallow(symbol_name))) {
+            symbol_name = tentative_name + "$" + i++;
+        }
+    }
+
+    if (!symbol_name) {
+        throw new Error("No symbol name could be generated in create_symbol()");
+    }
+
+    const symbol = make_node(SymClass, source, {
+        name: symbol_name,
+        scope
+    });
+
+    this.def_variable(symbol, init || null);
+
+    symbol.mark_enclosed();
+
+    return symbol;
+});
+
+
 AST_Node.DEFMETHOD("is_block_scope", return_false);
 AST_Class.DEFMETHOD("is_block_scope", return_false);
 AST_Lambda.DEFMETHOD("is_block_scope", return_false);
 AST_Toplevel.DEFMETHOD("is_block_scope", return_false);
 AST_SwitchBranch.DEFMETHOD("is_block_scope", return_false);
 AST_Block.DEFMETHOD("is_block_scope", return_true);
+AST_Scope.DEFMETHOD("is_block_scope", function () {
+    return this._block_scope || false;
+});
 AST_IterationStatement.DEFMETHOD("is_block_scope", return_true);
 
 AST_Lambda.DEFMETHOD("init_scope_vars", function() {
@@ -450,26 +643,19 @@ AST_Arrow.DEFMETHOD("init_scope_vars", function() {
     this.uses_arguments = false;
 });
 
-AST_Symbol.DEFMETHOD("mark_enclosed", function(options) {
+AST_Symbol.DEFMETHOD("mark_enclosed", function() {
     var def = this.definition();
     var s = this.scope;
     while (s) {
         push_uniq(s.enclosed, def);
-        if (options.keep_fnames) {
-            s.functions.forEach(function(d) {
-                if (keep_name(options.keep_fnames, d.name)) {
-                    push_uniq(def.scope.enclosed, d);
-                }
-            });
-        }
         if (s === def.scope) break;
         s = s.parent_scope;
     }
 });
 
-AST_Symbol.DEFMETHOD("reference", function(options) {
+AST_Symbol.DEFMETHOD("reference", function() {
     this.definition().references.push(this);
-    this.mark_enclosed(options);
+    this.mark_enclosed();
 });
 
 AST_Scope.DEFMETHOD("find_variable", function(name) {
@@ -481,7 +667,6 @@ AST_Scope.DEFMETHOD("find_variable", function(name) {
 AST_Scope.DEFMETHOD("def_function", function(symbol, init) {
     var def = this.def_variable(symbol, init);
     if (!def.init || def.init instanceof AST_Defun) def.init = init;
-    this.functions.set(symbol.name, def);
     return def;
 });
 
@@ -501,21 +686,35 @@ AST_Scope.DEFMETHOD("def_variable", function(symbol, init) {
 });
 
 function next_mangled(scope, options) {
+    let defun_scope;
+    if (
+        scopes_with_block_defuns
+        && (defun_scope = scope.get_defun_scope())
+        && scopes_with_block_defuns.has(defun_scope)
+    ) {
+        scope = defun_scope;
+    }
+
     var ext = scope.enclosed;
+    var nth_identifier = options.nth_identifier;
     out: while (true) {
-        var m = base54(++scope.cname);
-        if (RESERVED_WORDS.has(m)) continue; // skip over "do"
+        var m = nth_identifier.get(++scope.cname);
+        if (ALL_RESERVED_WORDS.has(m)) continue; // skip over "do"
 
         // https://github.com/mishoo/UglifyJS2/issues/242 -- do not
         // shadow a name reserved from mangling.
-        if (member(m, options.reserved)) continue;
+        if (options.reserved.has(m)) continue;
+
+        // Functions with short names might collide with base54 output
+        // and therefore cause collisions when keep_fnames is true.
+        if (unmangleable_names && unmangleable_names.has(m)) continue out;
 
         // we must ensure that the mangled name does not shadow a name
         // from some parent scope that is referenced in this or in
         // inner scopes.
-        for (var i = ext.length; --i >= 0;) {
-            var sym = ext[i];
-            var name = sym.mangled_name || (sym.unmangleable(options) && sym.name);
+        for (let i = ext.length; --i >= 0;) {
+            const def = ext[i];
+            const name = def.mangled_name || (def.unmangleable(options) && def.name);
             if (m == name) continue out;
         }
         return m;
@@ -527,10 +726,11 @@ AST_Scope.DEFMETHOD("next_mangled", function(options) {
 });
 
 AST_Toplevel.DEFMETHOD("next_mangled", function(options) {
-    var name;
+    let name;
+    const mangled_names = this.mangled_names;
     do {
         name = next_mangled(this, options);
-    } while (this.mangled_names.has(name));
+    } while (mangled_names.has(name));
     return name;
 });
 
@@ -568,12 +768,16 @@ AST_Symbol.DEFMETHOD("definition", function() {
 });
 
 AST_Symbol.DEFMETHOD("global", function() {
-    return this.definition().global;
+    return this.thedef.global;
 });
 
-AST_Toplevel.DEFMETHOD("_default_mangler_options", function(options) {
+/**
+ * Format the mangler options (if any) into their appropriate types
+ */
+export function format_mangler_options(options) {
     options = defaults(options, {
         eval        : false,
+        nth_identifier : base54,
         ie8         : false,
         keep_classnames: false,
         keep_fnames : false,
@@ -581,17 +785,21 @@ AST_Toplevel.DEFMETHOD("_default_mangler_options", function(options) {
         reserved    : [],
         toplevel    : false,
     });
-    if (options["module"]) {
-        options.toplevel = true;
+    if (options.module) options.toplevel = true;
+    if (!Array.isArray(options.reserved)
+        && !(options.reserved instanceof Set)
+    ) {
+        options.reserved = [];
     }
-    if (!Array.isArray(options.reserved)) options.reserved = [];
+    options.reserved = new Set(options.reserved);
     // Never mangle arguments
-    push_uniq(options.reserved, "arguments");
+    options.reserved.add("arguments");
     return options;
-});
+}
 
 AST_Toplevel.DEFMETHOD("mangle_names", function(options) {
-    options = this._default_mangler_options(options);
+    options = format_mangler_options(options);
+    var nth_identifier = options.nth_identifier;
 
     // We only need to mangle declaration nodes.  Special logic wired
     // into the code generator will display the mangled name if it's
@@ -600,7 +808,13 @@ AST_Toplevel.DEFMETHOD("mangle_names", function(options) {
     var lname = -1;
     var to_mangle = [];
 
-    var mangled_names = this.mangled_names = new Set();
+    if (options.keep_fnames) {
+        function_defs = new Set();
+    }
+
+    const mangled_names = this.mangled_names = new Set();
+    unmangleable_names = new Set();
+
     if (options.cache) {
         this.globals.forEach(collect);
         if (options.cache.props) {
@@ -618,6 +832,13 @@ AST_Toplevel.DEFMETHOD("mangle_names", function(options) {
             lname = save_nesting;
             return true;        // don't descend again in TreeWalker
         }
+        if (
+            node instanceof AST_Defun
+            && !(tw.parent() instanceof AST_Scope)
+        ) {
+            scopes_with_block_defuns = scopes_with_block_defuns || new Set();
+            scopes_with_block_defuns.add(node.parent_scope.get_defun_scope());
+        }
         if (node instanceof AST_Scope) {
             node.variables.forEach(collect);
             return;
@@ -626,9 +847,21 @@ AST_Toplevel.DEFMETHOD("mangle_names", function(options) {
             node.block_scope.variables.forEach(collect);
             return;
         }
+        if (
+            function_defs
+            && node instanceof AST_VarDef
+            && node.value instanceof AST_Lambda
+            && !node.value.name
+            && keep_name(options.keep_fnames, node.name.name)
+        ) {
+            function_defs.add(node.name.definition().id);
+            return;
+        }
         if (node instanceof AST_Label) {
-            var name;
-            do name = base54(++lname); while (RESERVED_WORDS.has(name));
+            let name;
+            do {
+                name = nth_identifier.get(++lname);
+            } while (ALL_RESERVED_WORDS.has(name));
             node.mangled_name = name;
             return true;
         }
@@ -637,21 +870,37 @@ AST_Toplevel.DEFMETHOD("mangle_names", function(options) {
             return;
         }
     });
+
     this.walk(tw);
-    to_mangle.forEach(function(def) { def.mangle(options); });
+
+    if (options.keep_fnames || options.keep_classnames) {
+        // Collect a set of short names which are unmangleable,
+        // for use in avoiding collisions in next_mangled.
+        to_mangle.forEach(def => {
+            if (def.name.length < 6 && def.unmangleable(options)) {
+                unmangleable_names.add(def.name);
+            }
+        });
+    }
+
+    to_mangle.forEach(def => { def.mangle(options); });
+
+    function_defs = null;
+    unmangleable_names = null;
+    scopes_with_block_defuns = null;
 
     function collect(symbol) {
-        if (!member(symbol.name, options.reserved)) {
-            if (!(symbol.export & MASK_EXPORT_DONT_MANGLE)) {
-                to_mangle.push(symbol);
-            }
+        if (symbol.export & MASK_EXPORT_DONT_MANGLE) {
+            unmangleable_names.add(symbol.name);
+        } else if (!options.reserved.has(symbol.name)) {
+            to_mangle.push(symbol);
         }
     }
 });
 
 AST_Toplevel.DEFMETHOD("find_colliding_names", function(options) {
-    var cache = options.cache && options.cache.props;
-    var avoid = new Set();
+    const cache = options.cache && options.cache.props;
+    const avoid = new Set();
     options.reserved.forEach(to_avoid);
     this.globals.forEach(add_def);
     this.walk(new TreeWalker(function(node) {
@@ -673,9 +922,12 @@ AST_Toplevel.DEFMETHOD("find_colliding_names", function(options) {
 });
 
 AST_Toplevel.DEFMETHOD("expand_names", function(options) {
-    base54.reset();
-    base54.sort();
-    options = this._default_mangler_options(options);
+    options = format_mangler_options(options);
+    var nth_identifier = options.nth_identifier;
+    if (nth_identifier.reset && nth_identifier.sort) {
+        nth_identifier.reset();
+        nth_identifier.sort();
+    }
     var avoid = this.find_colliding_names(options);
     var cname = 0;
     this.globals.forEach(rename);
@@ -687,22 +939,22 @@ AST_Toplevel.DEFMETHOD("expand_names", function(options) {
     function next_name() {
         var name;
         do {
-            name = base54(cname++);
-        } while (avoid.has(name) || RESERVED_WORDS.has(name));
+            name = nth_identifier.get(cname++);
+        } while (avoid.has(name) || ALL_RESERVED_WORDS.has(name));
         return name;
     }
 
     function rename(def) {
         if (def.global && options.cache) return;
         if (def.unmangleable(options)) return;
-        if (member(def.name, options.reserved)) return;
-        var d = def.redefined();
-        def.name = d ? d.name : next_name();
+        if (options.reserved.has(def.name)) return;
+        const redefinition = redefined_catch_def(def);
+        const name = def.name = redefinition ? redefinition.name : next_name();
         def.orig.forEach(function(sym) {
-            sym.name = def.name;
+            sym.name = name;
         });
         def.references.forEach(function(sym) {
-            sym.name = def.name;
+            sym.name = name;
         });
     }
 });
@@ -713,29 +965,38 @@ AST_Sequence.DEFMETHOD("tail_node", function() {
 });
 
 AST_Toplevel.DEFMETHOD("compute_char_frequency", function(options) {
-    options = this._default_mangler_options(options);
+    options = format_mangler_options(options);
+    var nth_identifier = options.nth_identifier;
+    if (!nth_identifier.reset || !nth_identifier.consider || !nth_identifier.sort) {
+        // If the identifier mangler is invariant, skip computing character frequency.
+        return;
+    }
+    nth_identifier.reset();
+
     try {
         AST_Node.prototype.print = function(stream, force_parens) {
             this._print(stream, force_parens);
             if (this instanceof AST_Symbol && !this.unmangleable(options)) {
-                base54.consider(this.name, -1);
+                nth_identifier.consider(this.name, -1);
             } else if (options.properties) {
-                if (this instanceof AST_Dot) {
-                    base54.consider(this.property, -1);
+                if (this instanceof AST_DotHash) {
+                    nth_identifier.consider("#" + this.property, -1);
+                } else if (this instanceof AST_Dot) {
+                    nth_identifier.consider(this.property, -1);
                 } else if (this instanceof AST_Sub) {
                     skip_string(this.property);
                 }
             }
         };
-        base54.consider(this.print_to_string(), 1);
+        nth_identifier.consider(this.print_to_string(), 1);
     } finally {
         AST_Node.prototype.print = AST_Node.prototype._print;
     }
-    base54.sort();
+    nth_identifier.sort();
 
     function skip_string(node) {
         if (node instanceof AST_String) {
-            base54.consider(node.value, -1);
+            nth_identifier.consider(node.value, -1);
         } else if (node instanceof AST_Conditional) {
             skip_string(node.consequent);
             skip_string(node.alternative);
@@ -745,10 +1006,11 @@ AST_Toplevel.DEFMETHOD("compute_char_frequency", function(options) {
     }
 });
 
-var base54 = (function() {
-    var leading = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ$_".split("");
-    var digits = "0123456789".split("");
-    var chars, frequency;
+const base54 = (() => {
+    const leading = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ$_".split("");
+    const digits = "0123456789".split("");
+    let chars;
+    let frequency;
     function reset() {
         frequency = new Map();
         leading.forEach(function(ch) {
@@ -758,19 +1020,20 @@ var base54 = (function() {
             frequency.set(ch, 0);
         });
     }
-    base54.consider = function(str, delta) {
+    function consider(str, delta) {
         for (var i = str.length; --i >= 0;) {
             frequency.set(str[i], frequency.get(str[i]) + delta);
         }
-    };
+    }
     function compare(a, b) {
         return frequency.get(b) - frequency.get(a);
     }
-    base54.sort = function() {
+    function sort() {
         chars = mergeSort(leading, compare).concat(mergeSort(digits, compare));
-    };
-    base54.reset = reset;
+    }
+    // Ensure this is in a usable initial state.
     reset();
+    sort();
     function base54(num) {
         var ret = "", base = 54;
         num++;
@@ -782,7 +1045,13 @@ var base54 = (function() {
         } while (num > 0);
         return ret;
     }
-    return base54;
+
+    return {
+        get: base54,
+        consider,
+        reset,
+        sort
+    };
 })();
 
 export {
