@@ -487,3 +487,44 @@ issue_2747: {
     }
     expect_stdout: "null 5 4"
 }
+
+if_return_same_value: {
+    options = {
+        conditionals: true,
+        if_return: true,
+        sequences: true,
+    }
+    input: {
+        function f() {
+            if (foo) {
+                return x();
+            }
+            if (bar) {
+                return x();
+            }
+        }
+        function g() {
+            if (foo) {
+                return x();
+            }
+            if (bar) {
+                return x();
+            }
+            return x();
+        }
+        function h() {
+            if (foo) {
+                return x();
+            }
+            if (bar) {
+                return x();
+            }
+            return y();
+        }
+    }
+    expect: {
+        function f() { return (foo || bar) ? x() : void 0 }
+        function g() { return (foo || bar), x() }
+        function h() { return (foo || bar) ? x() : y() }
+    }
+}
