@@ -332,3 +332,65 @@ big_int_scientific_format: {
         col: 8
     })
 }
+
+invalid_privatename_in_object: {
+    input: `
+        const myObject = {
+            foo: 'bar',
+            #something: 5,
+        }
+    `
+    expect_error: ({
+        name: "SyntaxError",
+        message: "private fields are not allowed in an object",
+        line: 4,
+        col: 12
+    })
+}
+
+private_field_out_of_class_field: {
+    input: `
+        function test() {
+            return this.#p;
+        }
+    `
+    expect_error: ({
+        name: "SyntaxError",
+        message: "Private field must be used in an enclosing class",
+        line: 3,
+        col: 24
+    })
+}
+
+private_field_out_of_class_field_in_operator: {
+    input: `
+        function test(input) {
+            #p in input;
+            return 10;
+        }
+    `
+    expect_error:({
+        name: "SyntaxError",
+        message: "Private field must be used in an enclosing class",
+        line: 3,
+        col: 12
+    })
+}
+
+invaild__in_operator_expression_in_class_field: {
+    input: `
+        class A {
+            #p;
+            isA () {
+                #p + 10;
+                return this.#p;
+            }
+        }
+    `
+    expect_error: ({
+        name: "SyntaxError",
+        message: "Unexpected token operator «+», expected operator «in»",
+        line: 5,
+        col: 19
+    })
+}
