@@ -7,6 +7,8 @@ import { Bar2 as kex, Baz as food } from "lel";
 
 const x = 0b01;
 let y = 6;
+import.meta;
+console.log(import.meta.url);
 
 export default x;
 export const z = 4;
@@ -69,6 +71,8 @@ var { w: w1, ...V } = { w: 7, x: 1, y: 2 };
 for (const x of y) {}
 async function f1() { await x; }
 
+const logicalExpression = 1 || 2;
+
 ``;
 `x`;
 `x${1}`;
